#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 17-7-8
# @Author  : JCai
# @desc    :

import os
import django

from utils.util import str_to_date

os.environ['DJANGO_SETTINGS_MODULE'] = 'EBayClub.settings'
django.setup()

from datetime import datetime, timedelta

import pytz
from django.utils import timezone
from info_sync.interface.trading_inter import MyEbay
from info_sync.models.listing_info_models import EBayListing, EBayListingInfoMore, EBayListingVariations
from utils.logger import logStatusBegin, logStatusEnd, logInfo
from info_sync.models.account_models import EBayAccount
from info_sync.tasks import listing_more_info_sync_task


def listing_base_selector_keys():
    return ['ItemArray.Item.ItemID',
            'ItemArray.Item.Site',
            'ItemArray.Item.SellingStatus.ListingStatus',
            'ItemArray.Item.ListingType',
            'ItemArray.Item.SKU',
            'ItemArray.Item.Title',
            'ItemArray.Item.SubTitle',
            'ItemArray.Item.Currency',
            'ItemArray.Item.SellingStatus.CurrentPrice',
            'ItemArray.Item.StartPrice',
            'ItemArray.Item.ReservePrice',
            'ItemArray.Item.BuyItNowPrice',
            'ItemArray.Item.ListingDuration',
            'ItemArray.Item.Quantity',
            'ItemArray.Item.SellingStatus.QuantitySold',
            'ItemArray.Item.ListingDetails.StartTime',
            'ItemArray.Item.ListingDetails.EndTime',
            'ItemArray.Item.Variations',
            'ItemArray.Item.Country',
            'ItemArray.Item.Location',
            # 'ItemArray.Item.PictureDetails.GalleryURL',
            # 'ItemArray.Item.ShippingDetails.ShippingServiceOptions',
            'PaginationResult']



def info_variation_change_to_sql(item_id, variations, open_date):
    variation_list = []
    variation_count = 0
    for variation in variations:
        VariationSpecifics = variation.get('VariationSpecifics', {}).get('NameValueList', [])
        if type(VariationSpecifics) == type({}):
            VariationSpecifics = [VariationSpecifics]
        seller_sku = variation.get('SKU', '')
        CurrentPrice = float(variation.get('StartPrice', {}).get('value', '0.0'))
        Quantity = int(variation.get('Quantity', '0'))
        QuantitySold = int(variation.get('SellingStatus', {}).get('QuantitySold', '0'))

        now_time = datetime.strptime(timezone.now().strftime('%Y-%m-%d'), '%Y-%m-%d')
        no_single_day = 0
        if QuantitySold == 0:
            start_time = datetime.strptime(open_date.strftime('%Y-%m-%d'), '%Y-%m-%d')
            no_single_day = (now_time - start_time).days

        variation_info = {}
        variation_listings = EBayListingVariations.objects.filter(ItemID=item_id, seller_sku=seller_sku)
        if variation_listings:
            variation_listing = variation_listings.first()
            if variation_listing:
                if CurrentPrice != variation_listing.CurrentPrice:
                    variation_info['CurrentPrice'] = CurrentPrice
                if Quantity != variation_listing.Quantity:
                    variation_info['Quantity'] = Quantity
                if QuantitySold != variation_listing.QuantitySold:
                    variation_info['QuantitySold'] = QuantitySold
                if VariationSpecifics != variation_listing.VariationSpecifics:
                    variation_info['VariationSpecifics'] = VariationSpecifics
                if no_single_day != variation_listing.no_single_day:
                    variation_info['no_single_day'] = no_single_day
                if QuantitySold != 0:
                    start_time = datetime.strptime(variation_listing.write_date.strftime('%Y-%m-%d'), '%Y-%m-%d')
                    day_temp = (now_time - start_time).days
                    if day_temp > 0:
                        if variation_listing.QuantitySold == QuantitySold:
                            no_single_day = variation_listing.QuantitySold + day_temp
                        else:
                            no_single_day = 0
                if no_single_day != variation_listing.no_single_day:
                    variation_info['no_single_day'] = no_single_day

                variation_listings.update(**variation_info)
                if len(variation_info):
                    variation_info['operate'] = 'update'
        else:
            variation_info['CurrentPrice'] = CurrentPrice
            variation_info['Quantity'] = Quantity
            variation_info['QuantitySold'] = QuantitySold
            variation_info['VariationSpecifics'] = VariationSpecifics
            variation_info['no_single_day'] = no_single_day
            EBayListingVariations.objects.create(
                ItemID=item_id,
                seller_sku=seller_sku,
                **variation_info
            )
            variation_info['operate'] = 'add'
        if len(variation_info):
            variation_info['ItemID'] = item_id
            variation_info['seller_sku'] = seller_sku
            variation_list.append(variation_info)
        variation_count += 1
    return variation_list, variation_count


def info_base_change_to_sql(item_id, item_info, store_id, variation_count, open_date):
    WatchCount = int(item_info.get('WatchCount', 0))
    Site = item_info.get('Site', '')
    ListingStatus = item_info.get('SellingStatus', {}).get('ListingStatus', '')
    ListingType = item_info.get('ListingType', '')
    seller_sku = item_info.get('SKU', '')
    Title = item_info.get('Title', '')
    currencyID = item_info.get('Currency', '')
    listing_price = float(item_info.get('SellingStatus', {}).get('CurrentPrice', {}).get('value', '0'))
    start_price = float(item_info.get('StartPrice', {}).get('value', '0'))
    reserve_price = float(item_info.get('ReservePrice', {}).get('value', '0'))
    buy_it_now_price = float(item_info.get('BuyItNowPrice', {}).get('value', '0'))
    ListingDuration = item_info.get('ListingDuration', '')
    quantity = int(item_info.get('Quantity', '0'))
    QuantitySold = int(item_info.get('SellingStatus', {}).get('QuantitySold', '0'))
    end_date = str_to_date(item_info.get('ListingDetails', {}).get('EndTime', ''))
    Country = item_info.get('Country', '')
    Location = item_info.get('Location', '')
    # image = item_info.get('PictureDetails', {}).get('GalleryURL', ''),

    # ShippingServiceCost = 0.0
    # _ShippingService = item_info.get('ShippingDetails', {}).get('ShippingServiceOptions', {})
    # if type(_ShippingService) == type({}):
    #     _ShippingService = [_ShippingService]
    # for __shipping_service in _ShippingService:
    #     _ShippingServicePriority = __shipping_service.get('ShippingServicePriority', '')
    #     if _ShippingServicePriority == '1':
    #         ShippingServiceCost = float(__shipping_service.get('ShippingServiceCost', {}).get('value', 0.0))
    #         break

    if variation_count:
        ListingType = 'Variation'
    no_single_day = 0
    listing_end_time = datetime.strptime(timezone.now().strftime('%Y-%m-%d'), '%Y-%m-%d')
    if QuantitySold == 0:
        start_time = datetime.strptime(open_date.strftime('%Y-%m-%d'), '%Y-%m-%d')
        no_single_day = (listing_end_time - start_time).days

    base_info = {}
    base_listings = EBayListing.objects.filter(ItemID=item_id)
    if base_listings:
        base_listing = base_listings.first()
        if base_listing:
            if WatchCount != base_listing.WatchCount:
                base_info['WatchCount'] = WatchCount
            if Site != base_listing.Site:
                base_info['Site'] = Site
            if ListingStatus != base_listing.ListingStatus:
                base_info['ListingStatus'] = ListingStatus
            if ListingType != base_listing.ListingType:
                base_info['ListingType'] = ListingType
            if seller_sku != base_listing.seller_sku:
                base_info['seller_sku'] = seller_sku
            if Title != base_listing.Title:
                base_info['Title'] = Title
            if currencyID != base_listing.currencyID:
                base_info['currencyID'] = currencyID
            if listing_price != base_listing.listing_price:
                base_info['listing_price'] = listing_price
            if start_price != base_listing.start_price:
                base_info['start_price'] = start_price
            if reserve_price != base_listing.reserve_price:
                base_info['reserve_price'] = reserve_price
            if buy_it_now_price != base_listing.buy_it_now_price:
                base_info['buy_it_now_price'] = buy_it_now_price
            if ListingDuration != base_listing.ListingDuration:
                base_info['ListingDuration'] = ListingDuration
            if quantity != base_listing.quantity:
                base_info['quantity'] = quantity
            if QuantitySold != base_listing.QuantitySold:
                base_info['QuantitySold'] = QuantitySold
            if open_date != base_listing.open_date:
                base_info['open_date'] = open_date
            if end_date != base_listing.end_date:
                base_info['end_date'] = end_date
            if Country != base_listing.Country:
                base_info['Country'] = Country
            if Location != base_listing.Location:
                base_info['Location'] = Location
            # if image != base_listing.image:
            #     base_info['image'] = image
            if variation_count != base_listing.variation:
                base_info['variation'] = variation_count
            # if ShippingServiceCost != base_listing.ShippingServiceCost:
            #     base_info['ShippingServiceCost'] = ShippingServiceCost
            if QuantitySold != 0:
                start_time = datetime.strptime(base_listing.write_date.strftime('%Y-%m-%d'), '%Y-%m-%d')
                day_temp = (listing_end_time - start_time).days
                if day_temp > 0:
                    if base_listing.QuantitySold == QuantitySold:
                        no_single_day = base_listing.QuantitySold + day_temp
                    else:
                        no_single_day = 0

            if no_single_day != base_listing.no_single_day:
                base_info['no_single_day'] = no_single_day

            base_listings.update(write_date=timezone.now(),**base_info)
            if len(base_info):
                base_info['operate'] = 'update'
                base_info['ItemID'] = item_id
    else:
        base_info['WatchCount'] = WatchCount
        base_info['Site'] = Site
        base_info['ListingStatus'] = ListingStatus
        base_info['ListingType'] = ListingType
        base_info['seller_sku'] = seller_sku
        base_info['Title'] = Title
        base_info['currencyID'] = currencyID
        base_info['listing_price'] = listing_price
        base_info['start_price'] = start_price
        base_info['reserve_price'] = reserve_price
        base_info['buy_it_now_price'] = buy_it_now_price
        base_info['ListingDuration'] = ListingDuration
        base_info['quantity'] = quantity
        base_info['QuantitySold'] = QuantitySold
        base_info['open_date'] = open_date
        base_info['end_date'] = end_date
        base_info['Country'] = Country
        base_info['Location'] = Location
        # base_info['image'] = image
        base_info['variation'] = variation_count
        base_info['ItemID'] = item_id
        base_info['ebay_account'] = store_id
        base_info['no_single_day'] = no_single_day
        base_info['source'] = 0
        # base_info['ShippingServiceCost'] = ShippingServiceCost
        EBayListing.objects.create(
            create_date=timezone.now(),
            write_date=timezone.now(),
            **base_info
        )
        EBayListingInfoMore.objects.create(ItemID=item_id)
        base_info['operate'] = 'add'
    return base_info


def listing_base_info_sync(store_obj, to_time, from_time):
    logStatusBegin('Listing Base Info Sync Begin')
    item_array = []
    store_name = store_obj.get('name', '')
    token = store_obj.get('Token', '')
    if not token or len(token) <= 0:
        logInfo('Sync error:\t%s\n\t%s' % (store_name, 'Token is Empty!'))
        logStatusEnd('Listing Base Info Sync END')
        return item_array

    store_id = store_obj.get('account_id', '')
    if not store_id or store_id <= 0:
        logInfo('sync error:\t%s\n\t%s' % (store_name, 'Store ID is Empty!'))
        logStatusEnd('Listing Base Info Sync END')
        return

    my_ebay = MyEbay({'Token': str(token), 'CertID': '', 'DevID': '', 'AppID': ''})
    call_data = {
        'EndTimeFrom': from_time,
        'IncludeWatchCount': True,
        'EndTimeTo': to_time,
        'IncludeVariations': True,
        'DetailLevel': 'ReturnAll',
        'OutputSelector': listing_base_selector_keys(),
        'Pagination': {'EntriesPerPage': 200},
        'GranularityLevel': 'Fine'
    }
    page = 1
    while True:
        try:
            logInfo('Sync Page: %s' % str(page))
            call_data['Pagination']['PageNumber'] = page
            status, message, listing_items, is_next_get = my_ebay.get_seller_list(call_data)
            if not status:
                logInfo('Sync Page: %s\t%s' % (store_name, message))
                break

            page += 1
            if type(listing_items) == type({}):
                listing_items = [listing_items]

            logInfo('Sync Page: Update Number\t%s' % str(len(listing_items)))
            for each in listing_items:
                item_id = each.get('ItemID', '')
                open_date = str_to_date(each.get('ListingDetails', {}).get('StartTime', ''))
                variations = each.get('Variations', {}).get('Variation', [])
                if type(variations) == type({}):  # if only have a variations  ,the resutl is a dict
                    variations = [variations]
                variation_list, variation_count = info_variation_change_to_sql(item_id, variations, open_date)
                info_base_change_to_sql(item_id, each, store_id, variation_count, open_date)

            if not is_next_get:
                logInfo('Sync Page: %s\t%s' % (store_name, message))
                break
        except Exception, e:
            logInfo('Sync ERROR:\t%s\n\t%s' % (store_name, str(e)))

    logStatusEnd('Listing Base Info Sync END')
    return item_array


def listing_more_selector_keys():
    return ['Item.ItemID',
            'Item.PrimaryCategory',
            'Item.SecondaryCategory',
            'Item.Storefront',
            # 'Item.VariationSpecificName'
            'Item.ConditionID',
            'Item.Description',
            'Item.PictureDetails.PictureURL',
            'Item.DispatchTimeMax',
            'Item.ShippingDetails.ShippingServiceOptions',
            'Item.ShippingDetails.InternationalShippingServiceOption',
            'Item.ShippingDetails.ExcludeShipToLocation',
            'Item.ReturnPolicy',
            'Item.BuyerRequirementDetails',
            'Item.ItemSpecifics.NameValueList',
            'Item.Variations',
            'Item.PaymentMethods',
            'Item.PayPalEmailAddress',
            'Item.PaymentInstructions',
            'Item.PostalCode']


def info_more_change_to_sql(item_id, item_info):
    description = item_info.get('Description', '')
    image_list = str(item_info.get('PictureDetails', {}).get('PictureURL', []))
    dispatch_time_max = int(item_info.get('DispatchTimeMax', '0'))
    shipping_domestic = str(item_info.get('ShippingDetails', {}).get('ShippingServiceOptions', []))
    shipping_international = str(item_info.get('ShippingDetails', {}).get('InternationalShippingServiceOption', []))
    exclude_ship_to_location = str(item_info.get('ShippingDetails', {}).get('ExcludeShipToLocation', []))
    return_policy = str(item_info.get('ReturnPolicy', []))
    buyer_requirement = str(item_info.get('BuyerRequirementDetails', []))
    category_specifics = str(item_info.get('ItemSpecifics', {}).get('NameValueList', []))
    payment_methods = str(item_info.get('PaymentMethods', []))
    paypal_email_address = item_info.get('PayPalEmailAddress', '')
    primary_category_id = item_info.get('PrimaryCategory', {}).get('CategoryID', '')
    secondary_category_id = item_info.get('SecondaryCategory', {}).get('CategoryID', '')
    store_category_id = item_info.get('Storefront', {}).get('StoreCategoryID', '')
    store_category_2id = item_info.get('Storefront', {}).get('StoreCategory2ID', '')
    primary_category_name = item_info.get('PrimaryCategory', {}).get('CategoryName', '')
    secondary_category_name = item_info.get('SecondaryCategory', {}).get('CategoryName', '')
    condition_id = item_info.get('ConditionID', '')
    more_info = {}
    more_listings = EBayListingInfoMore.objects.filter(item=item_id)
    if more_listings:
        more_listing = more_listings.first()
        if more_listing:
            if description != more_listing.description:
                more_info['description'] = description
            if image_list != more_listing.image_list:
                more_info['image_list'] = image_list
            if dispatch_time_max != more_listing.dispatch_time_max:
                more_info['dispatch_time_max'] = dispatch_time_max
            if shipping_domestic != more_listing.shipping_domestic:
                more_info['shipping_domestic'] = shipping_domestic
            if shipping_international != more_listing.shipping_international:
                more_info['shipping_international'] = shipping_international
            if exclude_ship_to_location != more_listing.exclude_ship_to_location:
                more_info['exclude_ship_to_location'] = exclude_ship_to_location
            if return_policy != more_listing.return_policy:
                more_info['return_policy'] = return_policy
            if buyer_requirement != more_listing.buyer_requirement:
                more_info['buyer_requirement'] = buyer_requirement
            if category_specifics != more_listing.category_specifics:
                more_info['category_specifics'] = category_specifics
            if payment_methods != more_listing.payment_methods:
                more_info['payment_methods'] = payment_methods
            if paypal_email_address != more_listing.paypal_email_address:
                more_info['paypal_email_address'] = paypal_email_address
            if primary_category_id != more_listing.primary_category_id:
                more_info['primary_category_id'] = primary_category_id
            if secondary_category_id != more_listing.secondary_category_id:
                more_info['secondary_category_id'] = secondary_category_id
            if store_category_id != more_listing.store_category_id:
                more_info['store_category_id'] = store_category_id
            if store_category_2id != more_listing.store_category_2id:
                more_info['store_category_2id'] = store_category_2id
            if primary_category_name != more_listing.primary_category_name:
                more_info['primary_category_name'] = primary_category_name
            if secondary_category_name != more_listing.secondary_category_name:
                more_info['secondary_category_name'] = secondary_category_name
            if condition_id != more_listing.condition_id:
                more_info['condition_id'] = condition_id
            more_listings.update(**more_info)
            if len(more_info):
                more_info['operate'] = 'update'
                more_info['item_id'] = item_id
    else:
        more_info['description'] = description
        more_info['image_list'] = image_list
        more_info['dispatch_time_max'] = dispatch_time_max
        more_info['shipping_domestic'] = shipping_domestic
        more_info['shipping_international'] = shipping_international
        more_info['exclude_ship_to_location'] = exclude_ship_to_location
        more_info['return_policy'] = return_policy
        more_info['buyer_requirement'] = buyer_requirement
        more_info['category_specifics'] = category_specifics
        more_info['payment_methods'] = payment_methods
        more_info['paypal_email_address'] = paypal_email_address
        more_info['primary_category_id'] = primary_category_id
        more_info['secondary_category_id'] = secondary_category_id
        more_info['store_category_id'] = store_category_id
        more_info['store_category_2id'] = store_category_2id
        more_info['primary_category_name'] = primary_category_name
        more_info['secondary_category_name'] = secondary_category_name
        more_info['condition_id'] = condition_id
        more_info['item'] = item_id
        EBayListingInfoMore.objects.create(
            **more_info
        )
        more_info['operate'] = 'add'

    return more_info


def listing_more_info_sync(item_ids, store_obj):
    logStatusBegin('Listing More Info Sync Begin')
    store_name = store_obj.get('name', '')
    token = store_obj.get('Token', '')
    if not token or len(token) <= 0:
        logInfo('sync error:\t%s\n\t%s' % (store_name, 'Token is Empty!'))
        logStatusEnd('Listing Base Info Sync END')
        return

    call_data = {
        'IncludeVariations': True,
        'IncludeItemSpecifics': True,
        'GranularityLevel': 'Fine',
        'DetailLevel': 'ReturnAll',
        'OutputSelector': listing_more_selector_keys(),
    }
    my_ebay = MyEbay({'Token': str(token), 'CertID': '', 'DevID': '', 'AppID': ''})
    count = len(item_ids)
    number = 1
    for item_id in item_ids:
        logInfo('%5d/%5d\tItemID:%s' % (number, count, item_id))
        try:
            status, message, listing_items = my_ebay.get_item(call_data)
            number += 1
            if not status:
                logInfo('sync info: %s\n%s' % (store_name, message))
                continue
            try:
                info_more_change_to_sql(item_id, listing_items)
            except Exception, e:
                logInfo('SQL error:\t%s\n\t%s' % (store_name, str(e)))

        except Exception, e:
            logInfo('sync error:\t%s\n\t%s' % (store_name, str(e)))
    logStatusEnd('Listing More Info Sync END')


def ebay_listing_sync(store):
    logStatusBegin('Listing Sync Begin:%s' % store.get('name', ''))
    from_time = str(datetime.now(pytz.UTC) - timedelta(days=3)).split('.')[0].replace(' ', 'T') + 'Z'
    to_time = str(datetime.now(pytz.UTC) + timedelta(days=33)).split('.')[0].replace(' ', 'T') + 'Z'
    # 获取新增的Listing的SKU
    item_array = listing_base_info_sync(store, to_time, from_time)
    # 更新Listing更多的信息
    # listing_more_info_sync_task.delay(item_array, store)
    logStatusEnd('Listing Sync End:%s' % store.get('name', ''))

